# This file created by Andrew Young<baram01@hotmail.com>
# For creating tac_plus database and tables

CREATE DATABASE tacacs;
USE tacacs;

# acl table :-
# id	        : ACL identification
# type          : client/host
# seq	        : Sequence number
# permission    : permit/deny rule in acl
# value	        : user/group id or client subnet
# value1        : if client acl, network of the client subnet
#                 if host acl, profile
# submask       : Subnet mask

CREATE TABLE acl ( id INT(4) NOT NULL, type INT(1) NOT NULL,
	seq INT(4) NOT NULL, permission INT(2) NOT NULL,
	value VARCHAR(20) NOT NULL, value1 REAL, submask REAL,
	PRIMARY KEY (id, type, seq) );

# attribute table :-
# id		: attribute ID
# name		: Name of attribute
# descr		: Description
# type		: Type of data (0 - string, 1 - integer, 2 - ipaddr, 3 - date)
# has_tag	: Attribute tagging
# encrypt	: Value of attribute needs to be encrypted
# disp_len	: Display length
# auth		: Type (0 - tacacs, 1 - radius)
# vid		: Vendor ID

CREATE TABLE attribute ( id INT NOT NULL, name VARCHAR(30),
	descr VARCHAR(255), type INT(1) NOT NULL, auth INT(1) NOT NULL,
	has_tag INT(1) DEFAULT 0, encrypt INT(1) DEFAULT 0,
	disp_len INT DEFAULT 0, vid INT(5) NOT NULL,
	PRIMARY KEY (id, vid, auth) );

INSERT INTO attribute VALUES (300, 'priv-lvl', 'Privilege Level', 1, 0, NULL, NULL, NULL, 0);
INSERT INTO attribute VALUES (300, 'foundry-privlvl', 'Privilege Level', 1, 0, NULL, NULL, NULL, 1991);
INSERT INTO attribute VALUES (300, 'junos-exec', 'Juniper Exec Service', 0, 0, NULL, NULL, NULL, 2636);

# vendor table :-
# id		: vendor id
# name		: name of vendor

CREATE TABLE vendor ( id INT(5) NOT NULL, name VARCHAR(30),
	PRIMARY KEY (id) );

INSERT INTO vendor VALUES ( 0, 'None' );
INSERT INTO vendor VALUES ( 9, 'Cisco' );
INSERT INTO vendor VALUES ( 52, 'Cabletron/Enterasys' );
INSERT INTO vendor VALUES ( 166, 'Shiva' );
INSERT INTO vendor VALUES ( 307, 'Livingston' );
INSERT INTO vendor VALUES ( 311, 'Microsoft' );
INSERT INTO vendor VALUES ( 429, 'Usr' );
INSERT INTO vendor VALUES ( 529, 'Ascend');
INSERT INTO vendor VALUES ( 1584, 'Bay-Networks' );
INSERT INTO vendor VALUES ( 1916, 'Extreme-Networks' );
INSERT INTO vendor VALUES ( 1991, 'Foundry-Networks' );
INSERT INTO vendor VALUES ( 2180, 'Versanet' );
INSERT INTO vendor VALUES ( 2352, 'Redback' );
INSERT INTO vendor VALUES ( 2636, 'Juniper' );
INSERT INTO vendor VALUES ( 3551, 'SprindTide' );
INSERT INTO vendor VALUES ( 4874, 'ERX' );
INSERT INTO vendor VALUES ( 8246, 'Cistron' );

# host table :-
# ip	     : IP V4 address/subnet bits of host or group name
# hostgroup  : Host group
# hkey	     : Decryption key
# enable     : Enable password for host that overrides all other enable passwords.
# prompt     : Banner to be displayed on host
# network    : Network of the IP
# submask    : Subnet mask
# loginacl   : Login ACL to allow/deny login access to devices
# enableacl  : Enable ACL to allow/deny enable access to devices
# host	     : 1 - host, 2 - host group
# vendor     : Vendor ID

CREATE TABLE host (ip varchar(20) NOT NULL PRIMARY KEY, 
	hostgroup varchar(20), hkey varchar(20), enable varchar(35),
	prompt TEXT, network REAL NOT NULL, submask REAL NOT NULL,
	loginacl INT(4), enableacl INT(4), host INT(1), vendor INT(5) DEFAULT 0,
	INDEX net (network,submask) );

# user table :-
# id	     : User/Group/Profile id
# uid        : User/Group/Profile username
# gid        : Group name (used if the data is a user otherwise NULL)
# groupid    : Group id
# comment    : Description about the user
# auth	     : Authentication method (1-local, 5-SecurID)
# password   : Password
# enable     : Enable password
# arap       : ARAP
# pap        : PAP
# chap       : CHAP
# mschap     : MSCHAP
# expires    : Expiration date and time
# disable    : Disable the account
# b_author   : Before authorization
# a_author   : After authorization
# svc_dflt   : Default behaviour for service
# cmd_dflt   : Default behaviour for command
# maxsess    : Maximum sessions
# user       : 1 - User, 2 - Group, 3 - Profile
# acl_id     : ACL that limits user/group to specific IP ranges
# sess       : Current number of sessions open
# shell      : Shell
# homedir    : Home Directory

CREATE TABLE user ( id REAL NOT NULL,
	uid varchar(20) NOT NULL, gid varchar(20), comment text,
	auth INT(1) DEFAULT 1, password varchar(35), enable varchar(35),
	arap varchar(35), pap varchar(35), chap varchar(35),
	mschap varchar(35), expires datetime, disable int(1) NOT NULL DEFAULT 0,
	b_author varchar(20), a_author varchar(20),
	svc_dflt int(1) NOT NULL DEFAULT 0,
	cmd_dflt int(1) NOT NULL DEFAULT 0,
	maxsess int(4), user int(1) NOT NULL DEFAULT 1, acl_id int(4),
	sess int(4), shell VARCHAR(255), homedir VARCHAR(255),
	PRIMARY KEY(uid, user) );

# contact_info :- ONLY USED FOR THE TRACKING USERS VIA WEB
# uid        : User id
# fname      : First name
# surname    : Surname (last name)
# address1   : Address
# address2   : 
# city       : City
# state      : State
# zip        : Zip code
# phone      : Telephone number
# email	     : Email

CREATE TABLE contact_info ( uid varchar(20) NOT NULL PRIMARY KEY,
	fname varchar(40) NOT NULL, surname varchar(40) NOT NULL,
	address1 varchar(40), address2 varchar(40), city varchar(30),
	state char(2), zip char(5), phone varchar(14), email varchar(100));

# node table :-
# id         : Node ID
# seq        : Sequence number
# service    : Service type (N_svc_cmd, N_svc_exec, N_svc_ppp, etc)
# type       : Type of node (N_arg, N_optarg, N_permit, N_deny, etc)
# value      : value of node
# value1     : value of node
# attr	     : Attribute/Command ID
# vid	     : Vendor ID

CREATE TABLE node ( id REAL NOT NULL, uid varchar(20) NOT NULL, 
	seq int(4) NOT NULL, service int(4) NOT NULL, type int(4),
	value varchar(50) NOT NULL, value1 varchar(50), attr INT,
	vid INT(5), INDEX id(id), INDEX uid(uid) );

# accounting table :-
# date       : Time stamp of occurance
# nas        : Network Access Server IP(eq. switch)
# uid        : User ID
# terminal   : Terminal used to connect to device
# client_ip  : Client IP
# type       : service type (start, stop, etc..)
# service    : service (exec, shell, etc..)
# priv_lvl   : Priviledge level (usefull in network device)
# cmd        : Command used
# elapsed_time : How much the user spent connected
# bytes_in : Incoming bytes to port
# bytes_out : Outgoing bytes from port
 
CREATE TABLE accounting( date datetime NOT NULL, nas varchar(16) NOT NULL,
	uid varchar(20) NOT NULL, terminal varchar(20),
	client_ip varchar(16) NOT NULL, type varchar(20), service varchar(20),
	priv_lvl INT(2), cmd varchar(255), elapsed_time INT(6),
	bytes_in INT(10), bytes_out INT(10), INDEX date_index(date),
	INDEX acct_index(uid), INDEX nas_index(nas),
	INDEX client_index(client_ip));

# access table :-
# date       : Time stamp of occurance
# nas        : Network Access Server IP(eq. switch)
# terminal   : Terminal used to connect to device
# uid        : User ID
# client_ip  : Client IP
# service    : service (login, enable, etc..)
# status     : rejected/accepted

CREATE TABLE access( date datetime NOT NULL, nas VARCHAR(16) NOT NULL,
	terminal VARCHAR(20), uid VARCHAR(20) NOT NULL,
	client_ip VARCHAR(16) NOT NULL, service VARCHAR(10), status VARCHAR(10),
	INDEX date_index(date), INDEX nas_index(nas), INDEX uid_index(uid),
	INDEX client_index(client_ip) );
 
# admin table :- ONLY USED FOR WEB
# uid		: User ID
# password	: Password
# priv_lvl	: Priviledge Level
# link		: Link to user table

CREATE TABLE admin ( uid varchar(20) NOT NULL PRIMARY KEY,
	password VARCHAR(35) NOT NULL, priv_lvl INT(2), link INT(1));

INSERT INTO admin VALUES ('admin',ENCRYPT('system'), 15, 0);

#create users needed to administrate tacacs
GRANT ALL ON tacacs.* TO tacacs@localhost IDENTIFIED BY 'tac_plus';
GRANT ALL ON tacacs.* TO tacacs IDENTIFIED BY 'tac_plus';

